from abc import ABC, abstractmethod


class BaseRepostitory(ABC):
    @abstractmethod
    def add(self, obj: object):
        raise NotImplementedError

    @abstractmethod
    def get(self):
        raise NotImplementedError

    @abstractmethod
    def list(self):
        raise NotImplementedError

    @abstractmethod
    def remove(self):
        raise NotImplementedError

    @abstractmethod
    def flush(self):
        raise NotImplementedError



import click

from francine.repositories.tinydb import TinyDBRepository
from francine.services.task import TaskService

repository = TinyDBRepository()
service = TaskService(repository)


@click.group()
def francine():
    """Francine helps you to track your short daily goals on track"""
    pass


@click.command()
@click.argument("description")
def add(description):
    """# adds a task for your list (francine add "task")"""
    service.flush()
    service.add(description)
    click.echo(f"Task added: {description}")


@click.command()
def get():
    """# shows the current task"""
    task = service.get()
    click.echo(f"Task: {task.description}")


@click.command()
def list():
    """# show all your last 24 hours task"""
    service.flush()
    tasks = service.list()
    for number, task in enumerate(tasks):
        click.echo(f"{number} - {task.description}")


@click.command()
def complete():
    """# completes the current task"""
    service.complete()
    click.echo("Task completed")


francine.add_command(add)
francine.add_command(get)
francine.add_command(list)
francine.add_command(complete)

if __name__ == "__main__":
    francine()



from francine.models.task import Task
from francine.repositories.tinydb import TinyDBRepository


class TaskService:
    def __init__(self, repository: TinyDBRepository):
        self.repository = repository

    def add(self, description: str):
        task = Task(description)
        self.repository.add(task)

    def get(self):
        return self.repository.get()

    def list(self):
        return self.repository.list()

    def complete(self):
        self.repository.remove()

    def flush(self):
        self.repository.flush()



from datetime import datetime, timedelta

from tinydb import Query, TinyDB

from francine.config import get_db_path
from francine.models.task import Task
from francine.repositories.base import BaseRepostitory

db_path = get_db_path()


class TinyDBRepository(BaseRepostitory):
    def __init__(self):
        self.db = TinyDB(db_path)
        self.query = Query()

    def add(self, task: Task):
        self.db.insert(
            {"description": task.description, "timestamp": task.timestamp.isoformat()}
        )

    def list(self):
        return [
            Task(item["description"], datetime.fromisoformat(item["timestamp"]))
            for item in self.db.all()
        ]

    def get(self):
        item = self.db.all()[0]
        return Task(item["description"], datetime.fromisoformat(item["timestamp"]))

    def remove(self):
        doc_id = self.db.all()[0].doc_id
        self.db.remove(
            doc_ids=[
                doc_id,
            ]
        )
        return [
            Task(item["description"], datetime.fromisoformat(item["timestamp"]))
            for item in self.db.all()
        ]

    def flush(self):
        diff = datetime.now() - timedelta(days=1)
        self.db.remove(self.query.timestamp < diff.isoformat())



import os

import platformdirs


def get_db_path():
    appname = "francine"
    appauthor = "gfabrizio"
    data_dir = platformdirs.user_data_dir(appname, appauthor)
    os.makedirs(data_dir, exist_ok=True)
    return os.path.join(data_dir, "francine.json")


db_path = get_db_path()
